import { VNode } from '../../stencil-public-runtime';
import type { ExtraNodeFuncConfig } from "../../types/index";
/**
 * Contains extra elements for stencil components.
 * Performs rendering and updates for external components.
 *
 * @example
 * In Plugins if you want to add extra elements to grid and use stenciljs vnodes reactivity:
 * function paginationPanel(this: PaginationPlugin, config: { refresh: () => void }) {
 *    // use `config.refresh()` for component to re-render
 *    return h('div')
 * }
 *
 * revogrid.registerVNode = [
 *    ...existingNodes,
 *     paginationPanel.bind(this)
 * ];


/**
 * @internal
 */
export declare class RevoGridExtra {
    /**
     * Nodes to render
     */
    nodes: (VNode | ((c: ExtraNodeFuncConfig) => VNode))[];
    /**
     * Force component to re-render
     */
    update: number;
    /**
     * Refreshes the extra component. Useful if you want to manually
     * force the component to re-render.
     */
    refresh(): Promise<void>;
    render(): any[];
}
