/**
 * Dispatches a custom event to a specified target element.
 *
 * @param target - The target element to dispatch the event to.
 * @param eventName - The name of the custom event.
 * @param detail - Optional. The detail of the custom event.
 * @returns The custom event that was dispatched.
 */
export declare function dispatch<DispatchDetail = any>(target: MouseEvent['target'], eventName: string, detail?: DispatchDetail): CustomEvent<DispatchDetail>;
/**
 * Dispatches a custom event based on an existing event object and prevents the default behavior of the original event.
 *
 * @param e - The original event object containing the target and preventDefault method.
 * @param eventName - The name of the custom event.
 * @param detail - Optional. The detail of the custom event.
 * @returns The custom event that was dispatched.
 */
export declare function dispatchByEvent<DispatchDetail = any>(e: Pick<MouseEvent, 'target' | 'preventDefault'>, // The original event object containing the target and preventDefault method.
eventName: string, // The name of the custom event.
detail?: DispatchDetail): CustomEvent;
