import type { ColumnData, ColumnGrouping, ColumnProp, ColumnRegular, ColumnTypes, DataType, DimensionCols, ViewSettingSizeProp } from "../types/index";
import { type Group as StoreGroup } from "../store/index";
export type ColumnItems = Record<DimensionCols, ColumnRegular[]>;
export interface ColumnGroup extends StoreGroup {
    level: number;
}
export type ColumnGroupingCollection = Record<DimensionCols, ColumnGroup[]>;
export declare function getCellData(val?: any): any;
export declare function getCellRaw(model?: DataType, column?: ColumnRegular): any;
export declare function getCellDataParsed(model: DataType, column: ColumnRegular): any;
/**
 * Column collection definition.
 * Used to access indexed data for columns.
 * Can be accessed via different events.
 */
export type ColumnCollection = {
    /**
     * Columns as they are in stores
     */
    columns: Record<DimensionCols, ColumnRegular[]>;
    /**
     * Columns indexed by prop for quick access, it's possible to have multiple columns with same prop but not recommended
     */
    columnByProp: Record<ColumnProp, ColumnRegular[]>;
    /**
     * Grouped columns
     */
    columnGrouping: ColumnGroupingCollection;
    /**
     * Max level of grouping
     */
    maxLevel: number;
    /**
     * Sorting
     */
    sort: Record<ColumnProp, ColumnRegular>;
};
/**
 * Get column type from column data
 */
export declare function getColumnType(rgCol: ColumnRegular): DimensionCols;
export declare function getColumnSizes(cols: ColumnRegular[]): ViewSettingSizeProp;
/**
 * Check if column is grouping column
 */
export declare function isColGrouping(colData: ColumnGrouping | ColumnRegular): colData is ColumnGrouping;
/**
 * This function is used to create a collection of columns.
 */
export declare function getColumns(columns: ColumnData, level?: number, types?: ColumnTypes): ColumnCollection;
export declare function gatherGroup<T extends ColumnCollection>(res: T, colData: ColumnGrouping, collection: T, level?: number): T;
export declare function getColumnByProp(columns: ColumnData, prop: ColumnProp): ColumnRegular | undefined;
